@@ -1,6 +1,6 @@
 /*
 	Audio File Library
-	Copyright (C) 1998, Michael Pruett <michael@68k.org>
+	Copyright (C) 1998-2000, Michael Pruett <michael@68k.org>
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -24,15 +24,15 @@
 	This file contains routines for parsing NeXT/Sun .snd format sound
 	files.
 */
+
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#include <config.h>
 #endif
 
 #include <assert.h>
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 
 #include "au.h"
@@ -41,21 +41,24 @@
 #include "util.h"
 #include "byteorder.h"
 
+extern struct _Compression g711_ulaw_compression;
+extern struct _Compression g711_alaw_compression;
+
 int _af_parseau (AFfilehandle file)
 {
 	u_int32_t	id, offset, length, encoding, sampleRate, channelCount;
 
 	assert(file != NULL);
-	assert(file->fp != NULL);
+	assert(file->fh != NULL);
 
-	fread(&id, 4, 1, file->fp);
+	af_fread(&id, 4, 1, file->fh);
 	assert(!memcmp(&id, ".snd", 4));
 
-	fread(&offset, 4, 1, file->fp);
-	fread(&length, 4, 1, file->fp);
-	fread(&encoding, 4, 1, file->fp);
-	fread(&sampleRate, 4, 1, file->fp);
-	fread(&channelCount, 4, 1, file->fp);
+	af_fread(&offset, 4, 1, file->fh);
+	af_fread(&length, 4, 1, file->fh);
+	af_fread(&encoding, 4, 1, file->fh);
+	af_fread(&sampleRate, 4, 1, file->fh);
+	af_fread(&channelCount, 4, 1, file->fh);
 
 	offset = BENDIAN_TO_HOST_INT32(offset);
 	length = BENDIAN_TO_HOST_INT32(length);
@@ -72,51 +75,72 @@ int _af_parseau (AFfilehandle file)
 	file->dataStart = offset;
 	file->trackBytes = length;
 
+	assert(file->compression == NULL);
+
 	switch (encoding)
 	{
-		case _AU_8BIT_ISDN_ULAW:
-		        /* u-law takes 1-byte input, but gives 2-byte output */
+		case _AU_FORMAT_MULAW_8:
 			file->sampleWidth = 16;
-			file->sampleFormat = AF_SAMPFMT_ISDNULAW; 
+			file->sampleFormat = AF_SAMPFMT_TWOSCOMP;
+			file->compression = &g711_ulaw_compression;
 			break;
-		case _AU_8BIT_PCM:
+		case _AU_FORMAT_ALAW_8:
+			file->sampleWidth = 16;
+			file->sampleFormat = AF_SAMPFMT_TWOSCOMP;
+			file->compression = &g711_alaw_compression;
+			break;
+		case _AU_FORMAT_LINEAR_8:
 			file->sampleWidth = 8;
-			file->sampleWidth = AF_SAMPFMT_TWOSCOMP;	/* ? */
+			file->sampleFormat = AF_SAMPFMT_TWOSCOMP;	/* maybe unsigned? */
 			break;
-		case _AU_16BIT_PCM:
+		case _AU_FORMAT_LINEAR_16:
 			file->sampleWidth = 16;
 			file->sampleFormat = AF_SAMPFMT_TWOSCOMP;
 			break;
-		case _AU_24BIT_PCM:
+		case _AU_FORMAT_LINEAR_24:
 			file->sampleWidth = 24;
 			file->sampleFormat = AF_SAMPFMT_TWOSCOMP;
 			break;
-		case _AU_32BIT_PCM:
+		case _AU_FORMAT_LINEAR_32:
 			file->sampleWidth = 32;
 			file->sampleFormat = AF_SAMPFMT_TWOSCOMP;
 			break;
-		case _AU_32BIT_FLOAT:
+		case _AU_FORMAT_FLOAT:
 			file->sampleWidth = 32;
 			file->sampleFormat = AF_SAMPFMT_FLOAT;
 			break;
-		case _AU_64BIT_FLOAT:
+		case _AU_FORMAT_DOUBLE:
 			file->sampleWidth = 64;
 			file->sampleFormat = AF_SAMPFMT_DOUBLE;
 			break;
+
 		default:
-			assert(0);
 			/*
 				This encoding method is not recognized or supported.
-				An appropriate error code should be returned.
+				A non-zero return value is returned.
+				Some sort of warning should be raised.
 			*/
+			return (-1);
 			break;
 	}
 
 	file->sampleRate = sampleRate;
 	file->channelCount = channelCount;
 	file->frameCount = length /
-		((file->sampleWidth / 8) * file->channelCount);
+		((file->sampleWidth + 7) / 8 * file->channelCount);
 
-	return 1; /* FIXME: return value never used */
+	if (file->compression != NULL &&
+		(file->compression->type == AF_COMPRESSION_G711_ULAW ||
+		file->compression->type == AF_COMPRESSION_G711_ALAW))
+	{
+		assert(file->channelCount > 0);
+		file->frameCount = length / file->channelCount;
+	}
+
+#ifdef DEBUG
+	_af_printfilehandle(file);
+#endif
+
+	/* A return value of zero indicates successful parsing. */
+	return 0;
 }
-
