@@ -1,6 +1,6 @@
 /*
 	Audio File Library
-	Copyright (C) 1998, Michael Pruett <michael@68k.org>
+	Copyright (C) 1998-2000, Michael Pruett <michael@68k.org>
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -41,12 +41,12 @@ int auWriteFrames (const AFfilehandle file, int track, void *samples,
 int auSyncFile (AFfilehandle file);
 
 static u_int32_t auencodingtype (AFfilehandle file);
-static void writeheader (AFfilehandle file);
+static void au_writeheader (AFfilehandle file);
 
 /* A return value of zero indicates successful synchronisation. */
 int auSyncFile (AFfilehandle file)
 {
-	writeheader(file);
+	au_writeheader(file);
 	return 0;
 }
 
@@ -58,7 +58,7 @@ int auWriteFrames (const AFfilehandle file, int track, void *samples,
 	if (file->dataStart == 0)
 	{
 		file->dataStart = 28;
-		writeheader(file);
+		au_writeheader(file);
 	}
 
 	if (file->compression != NULL)
@@ -75,7 +75,7 @@ int auWriteFrames (const AFfilehandle file, int track, void *samples,
 	else
 		frameCount = _af_blockWriteFrames(file, track, samples, count);
 
-	writeheader(file);
+	au_writeheader(file);
 
 #ifdef DEBUG
 	_af_printfilehandle(file);
@@ -84,36 +84,7 @@ int auWriteFrames (const AFfilehandle file, int track, void *samples,
 	return frameCount;
 }
 
-#if 0
-static int writeframes (AFfilehandle file, int track, void *samples, const int count)
-{
-	int	frameSize;
-
-	frameSize = (file->sampleWidth / 8) * file->channelCount;
-
-	af_fseek(file->fh, file->dataStart, SEEK_SET);
-	af_fseek(file->fh, file->currentFrame * frameSize, SEEK_CUR);
-
-	if (file->virtualByteOrder == AF_BYTEORDER_BIGENDIAN)
-	{
-		af_fwrite(samples, 1, frameSize * count, file->fh);
-	}
-	else
-	{
-		if (file->sampleWidth == 8)
-			af_fwrite(samples, 1, frameSize * count, file->fh);
-		else if (file->sampleWidth == 16)
-			writeswapblock16(file->fh, samples, file->channelCount * count);
-		else if (file->sampleWidth == 32)
-			writeswapblock32(file->fh, samples, file->channelCount * count);
-	}
-
-	file->frameCount += count;
-	file->currentFrame = file->frameCount;
-}
-#endif
-
-static void writeheader (AFfilehandle file)
+static void au_writeheader (AFfilehandle file)
 {
 	struct _AUHeader	auheader;
 	int					frameSize;
@@ -121,7 +92,8 @@ static void writeheader (AFfilehandle file)
 	frameSize = (file->sampleWidth + 7) / 8 * file->channelCount;
 
 	if (file->compression != NULL &&
-		file->compression->type == AF_COMPRESSION_G711_ULAW)
+		(file->compression->type == AF_COMPRESSION_G711_ULAW ||
+		file->compression->type == AF_COMPRESSION_G711_ALAW))
 		frameSize = file->channelCount;
 
 	memcpy(auheader.id, ".snd", 4);
@@ -146,26 +118,28 @@ static u_int32_t auencodingtype (AFfilehandle file)
 {
 	u_int32_t	encoding = 0;
 
-	if (file->compression != NULL &&
-		file->compression->type == AF_COMPRESSION_G711_ULAW)
+	if (file->compression != NULL)
 	{
-		encoding = _AU_8BIT_G711_ULAW;
+		if (file->compression->type == AF_COMPRESSION_G711_ULAW)
+			encoding = _AU_FORMAT_MULAW_8;
+		else if (file->compression->type == AF_COMPRESSION_G711_ALAW)
+			encoding = _AU_FORMAT_ALAW_8;
 	}
-	else if (file->sampleFormat == AF_SAMPFMT_DOUBLE)
-		encoding = _AU_64BIT_FLOAT;
-	else if (file->sampleFormat == AF_SAMPFMT_FLOAT)
-		encoding = _AU_32BIT_FLOAT;
 	else if (file->sampleFormat == AF_SAMPFMT_TWOSCOMP)
 	{
-		if (file->sampleWidth == 32)
-			encoding = _AU_32BIT_PCM;
-		else if (file->sampleWidth == 24)
-			encoding = _AU_24BIT_PCM;
+		if (file->sampleWidth == 8)
+			encoding = _AU_FORMAT_LINEAR_8;
 		else if (file->sampleWidth == 16)
-			encoding = _AU_16BIT_PCM;
-		else if (file->sampleWidth == 8)
-			encoding = _AU_8BIT_PCM;
+			encoding = _AU_FORMAT_LINEAR_16;
+		else if (file->sampleWidth == 24)
+			encoding = _AU_FORMAT_LINEAR_24;
+		else if (file->sampleWidth == 32)
+			encoding = _AU_FORMAT_LINEAR_32;
 	}
+	else if (file->sampleFormat == AF_SAMPFMT_FLOAT)
+		encoding = _AU_FORMAT_FLOAT;
+	else if (file->sampleFormat == AF_SAMPFMT_DOUBLE)
+		encoding = _AU_FORMAT_DOUBLE;
 
 	return encoding;
 }
