@@ -1,6 +1,6 @@
 /*
 	Audio File Library
-	Copyright (C) 1998, Michael Pruett <michael@68k.org>
+	Copyright (C) 1998-2000, Michael Pruett <michael@68k.org>
 
 	This library is free software; you can redistribute it and/or
 	modify it under the terms of the GNU Library General Public
@@ -23,25 +23,24 @@
 
 	This file contains routines which facilitate writing to WAVE files.
 */
+
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#include <config.h>
 #endif
 
 #include <sys/types.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <unistd.h>
 #include <string.h>
 #include <assert.h>
 
 #include "audiofile.h"
 #include "afinternal.h"
 #include "byteorder.h"
-#include "swapblock.h"
 #include "wave.h"
 #include "util.h"
+#include "block.h"
 
-int _af_blockWriteFrames (AFfilehandle file, int track, void *samples, int frameCount);
 int waveWriteFrames (const AFfilehandle file, int track, void *samples,
 	const int count);
 int waveSyncFile (AFfilehandle file);
@@ -50,10 +49,6 @@ static void WriteFormat (const AFfilehandle file);
 static int WriteData (const AFfilehandle file, int track, void *samples,
 	const int count);
 
-/*
-	PCM is the only format that is supported at present.
-	Logarithmic encoding sucks a fat dick.
-*/
 static void WriteFormat (const AFfilehandle file)
 {
 	u_int16_t	formatTag, channelCount;
@@ -63,32 +58,45 @@ static void WriteFormat (const AFfilehandle file)
 
 	assert(file != NULL);
 
-	fwrite("fmt ", 4, 1, file->fp);
+	af_fwrite("fmt ", 4, 1, file->fh);
 	chunkSize = 16;
 	chunkSize = HOST_TO_LENDIAN_INT32(chunkSize);
-	fwrite(&chunkSize, 4, 1, file->fp);
+	af_fwrite(&chunkSize, 4, 1, file->fh);
+
+	if (file->compression != NULL)
+	{
+		if (file->compression->type == AF_COMPRESSION_G711_ULAW)
+			formatTag = WAVE_FORMAT_MULAW;
+		else if (file->compression->type == AF_COMPRESSION_G711_ALAW)
+			formatTag = WAVE_FORMAT_ALAW;
+		/*
+			We should raise an error about an unsupported
+			compression type.
+		*/
+	}
+	else
+		formatTag = WAVE_FORMAT_PCM;
 
-	formatTag = WAVE_FORMAT_PCM;
 	formatTag = HOST_TO_LENDIAN_INT16(formatTag);
-	fwrite(&formatTag, 2, 1, file->fp);
+	af_fwrite(&formatTag, 2, 1, file->fh);
 	formatTag = LENDIAN_TO_HOST_INT16(formatTag);
 
 	channelCount = file->channelCount;
 	channelCount = HOST_TO_LENDIAN_INT16(channelCount);
-	fwrite(&channelCount, 2, 1, file->fp);
+	af_fwrite(&channelCount, 2, 1, file->fh);
 
 	sampleRate = file->sampleRate;
 	sampleRate = HOST_TO_LENDIAN_INT32(sampleRate);
-	fwrite(&sampleRate, 4, 1, file->fp);
+	af_fwrite(&sampleRate, 4, 1, file->fh);
 
 	averageBytesPerSecond =
 		file->channelCount * file->sampleRate * (file->sampleWidth / 8);
 	averageBytesPerSecond = HOST_TO_LENDIAN_INT32(averageBytesPerSecond);
-	fwrite(&averageBytesPerSecond, 4, 1, file->fp);
+	af_fwrite(&averageBytesPerSecond, 4, 1, file->fh);
 
 	blockAlign = file->channelCount * (file->sampleWidth / 8);
 	blockAlign = HOST_TO_LENDIAN_INT16(blockAlign);
-	fwrite(&blockAlign, 2, 1, file->fp);
+	af_fwrite(&blockAlign, 2, 1, file->fh);
 
 	if (formatTag == WAVE_FORMAT_PCM)
 	{
@@ -96,7 +104,15 @@ static void WriteFormat (const AFfilehandle file)
 
 		bitsPerSample = file->sampleWidth;
 		bitsPerSample = HOST_TO_LENDIAN_INT16(bitsPerSample);
-		fwrite(&bitsPerSample, 2, 1, file->fp);
+		af_fwrite(&bitsPerSample, 2, 1, file->fh);
+	}
+	else if (formatTag == WAVE_FORMAT_MULAW || formatTag == WAVE_FORMAT_ALAW)
+	{
+		u_int16_t	bitsPerSample;
+
+		bitsPerSample = 16;
+		bitsPerSample = HOST_TO_LENDIAN_INT16(bitsPerSample);
+		af_fwrite(&bitsPerSample, 2, 1, file->fh);
 	}
 }
 
@@ -105,6 +121,7 @@ static int WriteData (const AFfilehandle file, int track, void *samples,
 	const int count)
 {
 	u_int32_t	frameSize, chunkSize;
+	int		frameCount = 0;
 
 	assert(file);
 	assert(samples);
@@ -119,14 +136,28 @@ static int WriteData (const AFfilehandle file, int track, void *samples,
 
 	if (file->dataStart == 0)
 	{
-		fwrite("data", 4, 1, file->fp);
-		fwrite(&chunkSize, 4, 1, file->fp);
-		file->dataStart = ftell(file->fp);
+		af_fwrite("data", 4, 1, file->fh);
+		af_fwrite(&chunkSize, 4, 1, file->fh);
+		file->dataStart = af_ftell(file->fh);
 	}
 
-	fseek(file->fp, file->dataStart + file->currentFrame * frameSize, SEEK_SET);
+	af_fseek(file->fh, file->dataStart + file->currentFrame * frameSize, SEEK_SET);
 
-	return _af_blockWriteFrames(file, track, samples, count);
+	if (file->compression != NULL)
+	{
+		AFframecount	(*writeFrames)(AFfilehandle, int, void *, const int);
+
+		assert(file->compression->codec);
+
+		writeFrames = file->compression->codec->writeFrames;
+
+		assert(writeFrames);
+
+		frameCount = writeFrames(file, track, samples, count);
+	}
+	frameCount = _af_blockWriteFrames(file, track, samples, count);
+
+	return frameCount;
 }
 
 int waveWriteFrames (const AFfilehandle file, int track, void *samples,
@@ -138,9 +169,9 @@ int waveWriteFrames (const AFfilehandle file, int track, void *samples,
 	{
 		u_int32_t	zero = 0;
 
-		fwrite("RIFF", 4, 1, file->fp);
-		fwrite(&zero, 4, 1, file->fp);
-		fwrite("WAVE", 4, 1, file->fp);
+		af_fwrite("RIFF", 4, 1, file->fh);
+		af_fwrite(&zero, 4, 1, file->fh);
+		af_fwrite("WAVE", 4, 1, file->fh);
 
 		WriteFormat(file);
 	}
@@ -156,20 +187,30 @@ int waveSyncFile (AFfilehandle file)
 	{
 		size_t	dataLength, fileLength;
 
-		dataLength = file->frameCount * file->channelCount *
-			((file->sampleWidth + 7) / 8);
+		/*
+			If the file has one of the supported compression types,
+			calculate the extent of the audio track data specially.
+		*/
+		if (file->compression != NULL &&
+			(file->compression->type == AF_COMPRESSION_G711_ULAW ||
+			file->compression->type == AF_COMPRESSION_G711_ALAW))
+				dataLength = file->frameCount * file->channelCount;
+		/* Otherwise the file has normal PCM data. */
+		else
+			dataLength = file->frameCount * file->channelCount *
+				((file->sampleWidth + 7) / 8);
 
 		dataLength = HOST_TO_LENDIAN_INT32(dataLength);
-		fseek(file->fp, file->dataStart - 4, SEEK_SET);
-		fwrite(&dataLength, 4, 1, file->fp);
+		af_fseek(file->fh, file->dataStart - 4, SEEK_SET);
+		af_fwrite(&dataLength, 4, 1, file->fh);
 
-		fseek(file->fp, 0, SEEK_END);
-		fileLength = ftell(file->fp);
+		af_fseek(file->fh, 0, SEEK_END);
+		fileLength = af_ftell(file->fh);
 		fileLength -= 8;
 		fileLength = HOST_TO_LENDIAN_INT32(fileLength);
 
-		fseek(file->fp, 4, SEEK_SET);
-		fwrite(&fileLength, 4, 1, file->fp);
+		af_fseek(file->fh, 4, SEEK_SET);
+		af_fwrite(&fileLength, 4, 1, file->fh);
 	}
 
 	return 0;
