@@ -85,11 +85,18 @@ int main (int argc, char **argv)
 			}
 
 			i++;
+		} else {
+			printf("Unrecognized command %s\n", argv[i]);
 		}
 
 		i++;
 	}
 
+	if(format < 0) {
+		printf("You must specify an output format ('format <fmtname>' as a command)\n");
+		return 1;
+	}
+
 	infile = afOpenFile(infilename, "r", NULL);
 
 	frameCount = afGetFrameCount(infile, AF_DEFAULT_TRACK);
