@@ -18,8 +18,9 @@
 	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 	MA 02111-1307, USA.
 */
+
 #ifdef HAVE_CONFIG_H
-#  include <config.h>
+#include <config.h>
 #endif
 
 #include <stdio.h>
@@ -29,7 +30,7 @@
 #ifdef __USE_SGI_HEADERS__
 #include <dmedia/audiofile.h>
 #else
-#include "audiofile.h"
+#include <audiofile.h>
 #endif
 
 char copyright[] = "1998 Michael Pruett";
@@ -38,7 +39,7 @@ char name[] = "Michael Pruett's home-brew methamphetamines";
 /* Two frames of 16-bit stereo samples. */
 u_int16_t data[] = {0, 1, 2, 3};
 
-int main (int argc, char *argv[])
+int main (int argc, char **argv)
 {
 	AFfilehandle	file;
 	int				*miscids;
